﻿//------------------------------------------------------------------------------------------
// <copyright file="MyModuleConfiguration.cs" company="seca gmbh &amp; co. kg">
//      Copyright © seca gmbh &amp; co. kg 2010-2011. All rights reserved.
// </copyright>
//------------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;
using MyModule.Properties;
using Seca.Framework.Core;
using Seca.Framework.Core.GuiAttributes;

namespace MyModule
{
	/// <summary>
	/// Configuration data used by MyModule
	/// </summary>
	public class MyModuleConfiguration : PropertyBag
	{
		private bool isDataLoaded;
		protected override void OnDataLoaded()
		{
			base.OnDataLoaded();
			if (string.IsNullOrEmpty(this.password))
				this.SetError("Password", "Please provide a password"); // This will be cleared on successful entry
			this.isDataLoaded = true;
		}

		private const string firstNamesMaleDefault = "Joe;Jim;Harry;Frank;Barack;George";
		private string firstNamesMale = firstNamesMaleDefault;

		[DefaultValue(firstNamesMaleDefault)]
		public string FirstNamesMale
		{
			get { return firstNamesMale; }
			set
			{
				SetProperty(
							  () =>
							  {
								  checkNameList(value);
								  firstNamesMale = value;
							  },
							  "FirstNamesMale");
			}
		}

		private static void checkNameList(string value)
		{
			try
			{
				string[] a = value.Split(';');
				if (string.IsNullOrEmpty(a[0]))
					throw new ArgumentException();
			}
			catch
			{
				throw new SecaException(
					string.Format(
						Resources.MyModuleConfiguration_checkNameList___0___is_not_a_list_of_names,
						value));
			}
		}

		private const string firstNamesFemaleDefault = "Anne;Rosa;Kirsten;Lilly;Madeleine;Hillary";
		private string firstNamesFemale = firstNamesFemaleDefault;

		[DefaultValue(firstNamesFemaleDefault)]
		public string FirstNamesFemale
		{
			get { return firstNamesFemale; }
			set
			{
				SetProperty(
								() =>
								{
									checkNameList(value);
									firstNamesFemale = value;
								},
								"FirstNamesFemale");
			}
		}

		private const string surnamesDefault = "Miller;Tailor;Allan;Obama;Bush;Lindberg;Rose;Parker";
		private string surnames = surnamesDefault;

		[DefaultValue(surnamesDefault)]
		public string Surnames
		{
			get { return surnames; }
			set
			{
				SetProperty(
								() =>
								{
									checkNameList(value);
									surnames = value;
								},
								"Surnames");
			}
		}

		private bool enableSelectedSurname;

		public bool EnableSelectedSurname
		{
			get { return this.enableSelectedSurname; }
			set { SetProperty(() => enableSelectedSurname = value, "EnableSelectedSurname"); }
		}

		private string selectedSurname;

		[GuiEnable("EnableSelectedSurname")]
		[GuiComboBox("SurnameList")]
		public string SelectedSurname
		{
			get { return selectedSurname; }
			set { SetProperty(() => selectedSurname = value, "SelectedSurname"); }
		}

		public IEnumerable<string> SurnameList { get { return Surnames.Split(';'); } }

		public class MyDropdownItem
		{
			public double MyNumber { get; private set; }
			private readonly string display;
			public MyDropdownItem(double number, string display)
			{
				this.MyNumber = number;
				this.display = display;
			}

			public override string ToString()
			{
				return display;
			}
		}

		private double number;
		[GuiComboBox("NumberList", "MyNumber")]
		public double Number
		{
			get { return number; }
			set { SetProperty(() => number = value, "Number"); }
		}

		private readonly IEnumerable<MyDropdownItem> numberList = new[] 
		{ 
			new MyDropdownItem(0, "Zero"),
			new MyDropdownItem(0.25, "Quarter"),
			new MyDropdownItem(1, "One"),
			new MyDropdownItem(Math.PI, "Pi"),
			new MyDropdownItem(2011, "MMXI")
		};

		public IEnumerable<MyDropdownItem> NumberList
		{
			get { return this.numberList; }
		}

		private string workingDirectory;
		/// <summary>
		/// Gets or sets the working directory of the module
		/// </summary>
		[GuiFolder]
		public string WorkingDirectory
		{
			get { return workingDirectory; }
			set { SetProperty(() => workingDirectory = value, "WorkingDirectory"); }
		}

		/// <summary>
		/// Sample Options
		/// </summary>
		public enum MyOptions { One, Two, Three }

		private MyOptions options;
		[GuiIgnore]
		public MyOptions Options
		{
			get { return options; }
			set { SetProperty(() => options = value, "OptionOne", "OptionTwo", "OptionThree"); }
		}

		[XmlIgnore]
		[GuiRadioButton]
		public bool OptionOne
		{
			get { return Options == MyOptions.One; }
			set { if (value) Options = MyOptions.One; }
		}

		[XmlIgnore]
		[GuiRadioButton]
		public bool OptionTwo
		{
			get { return Options == MyOptions.Two; }
			set { if (value) Options = MyOptions.Two; }
		}

		[XmlIgnore]
		[GuiRadioButton]
		public bool OptionThree
		{
			get { return Options == MyOptions.Three; }
			set { if (value) Options = MyOptions.Three; }
		}

		private string password;

		[GuiPassword]
		public string Password
		{
			get { return password; }
			set
			{
				SetProperty(
					() =>
					{
						password = value;
						if (isDataLoaded)
						{
							if (string.IsNullOrEmpty(this.password))
								throw new SecaException(Resources.MyModuleConfiguration_Password_Please_provide_a_password_);
							this.SetError("PasswordRepeated", Resources.MyModuleConfiguration_Password_Please_repeat_the_password); // Flag repeated password on password user entry
						}
					},
						"Password");
			}
		}

		private string passwordRepeated;

		[GuiPassword]
		[XmlIgnore]
		public string PasswordRepeated
		{
			get { return passwordRepeated; }
			set
			{
				SetProperty(
					() =>
					{
						passwordRepeated = value;
						if (this.password != this.passwordRepeated)
						{
							throw new SecaException(Resources.MyModuleConfiguration_PasswordRepeated_The_password_and_the_repeated_password_don_t_match_);
						}
					},
					"PasswordRepeated");
			}
		}

		public override string GetCaption(string propertyName)
		{
			return
				Resources.ResourceManager.GetString(propertyName)
				?? base.GetCaption(propertyName);
		}
	}
}
